package bitcask

import (
	"time"

	"github.com/prologic/bitcask"

	"git.tcp.direct/kayos/tr4proute-dhcp/handler"
	"git.tcp.direct/kayos/tr4proute-dhcp/logger"
	"git.tcp.direct/kayos/tr4proute-dhcp/plugins"
	"github.com/insomniacslk/dhcp/dhcpv4"
	"github.com/insomniacslk/dhcp/dhcpv6"
)

var log = logger.GetLogger("plugins/bitcask")

var db4 *bitcask.Bitcask
var db6 *bitcask.Bitcask

// Plugin wraps plugin registration information
var Plugin = plugins.Plugin{
	Name: "bitcask",
	Setup6:  setup6,
	Setup4:  setup4,
}

// Handler6 handles DHCPv6 packets for the file plugin
func Handler6(req, resp dhcpv6.DHCPv6) (dhcpv6.DHCPv6, bool) {
	m, err := req.GetInnerMessage()
	if err != nil {
		log.Error().Err(err).Msg("BUG: could not decapsulate")
		return nil, true
	}

	if m.Options.OneIANA() == nil {
		log.Debug().Msg("No address requested")
		return resp, false
	}

	mac, err := dhcpv6.ExtractMAC(req)
	if err != nil {
		log.Warn().Msg("Could not find client MAC, passing")
		return resp, false
	}

	log.Debug().Str("MAC", mac.String()).Msg("LOOKUP_IP")

	if !db6.Has(mac) {
		log.Warn().Str("MAC", mac.String()).Msg("MAC address unknown")
		return resp, false
	}

	ipaddr, _ := db6.Get(mac)

	log.Debug().IPAddr("IP_ADDR", ipaddr).Str("MAC", mac.String()).Msg("IP_FOUND")

	resp.AddOption(&dhcpv6.OptIANA{
		IaId: m.Options.OneIANA().IaId,
		Options: dhcpv6.IdentityOptions{Options: []dhcpv6.Option{
			&dhcpv6.OptIAAddress{
				IPv6Addr:          ipaddr,
				PreferredLifetime: 3600 * time.Second,
				ValidLifetime:     3600 * time.Second,
			},
		}},
	})
	return resp, false
}

// Handler4 handles DHCPv4 packets for the file plugin
func Handler4(req, resp *dhcpv4.DHCPv4) (*dhcpv4.DHCPv4, bool) {
	// placeholder
}

func setup6(args ...string) (handler.Handler6, error) {
	h6, _, err := setupBitcask(true, args...)
	return h6, err
}

func setup4(args ...string) (handler.Handler4, error) {
	_, h4, err := setupBitcask(false, args...)
	return h4, err
}

func setupBitcask(v6 bool, args ...string) (handler.Handler6, handler.Handler4, error) {
	var err error
	var db *bitcask.Bitcask

	switch v6 {
	case false:
		db4 = db.Open(datapath + "leasesv6")
	case true:
		db6 = db.Open(datapath + "leasesv4")
	}

	log.Info().Int("count", len(records)).Str("filename", filename).Msg("successfully loaded leases from file")
	return Handler6, Handler4, nil
}
